@@ -71,9 +71,7 @@ class Analyze(Check):
             self._command_arguments.targets,
             self._configuration,
             self._original_directory,
-            self._configuration.project_root,
             filter_directory=self._command_arguments.filter_directory,
-            buck_mode=self._command_arguments.buck_mode,
             isolate=True,
         )
 
