@@ -6,6 +6,7 @@
 import dataclasses
 import json
 import logging
+import os
 from pathlib import Path
 from typing import List
 
@@ -15,7 +16,7 @@ from ... import (
     configuration as configuration_module,
     error,
 )
-from . import server_connection, start
+from . import server_connection, start, remote_logging
 
 
 LOG: logging.Logger = logging.getLogger(__name__)
@@ -56,7 +57,7 @@ def parse_type_error_response(response: str) -> List[error.Error]:
 
 
 def _relativize_error_path(error: error.Error) -> error.Error:
-    relativized_path = error.path.relative_to(Path.cwd())
+    relativized_path = os.path.relpath(str(error.path), str(Path.cwd()))
     return dataclasses.replace(error, path=relativized_path)
 
 
@@ -106,6 +107,7 @@ def run_incremental(
         _show_progress_and_display_type_errors(log_path, socket_path, output)
 
 
+@remote_logging.log_usage(command_name="incremental")
 def run(
     configuration: configuration_module.Configuration,
     incremental_arguments: command_arguments.IncrementalArguments,
@@ -114,6 +116,4 @@ def run(
         run_incremental(configuration, incremental_arguments)
         return commands.ExitCode.SUCCESS
     except Exception as error:
-        raise commands.ClientException(
-            f"Exception occured during incremental query: {error}"
-        ) from error
+        raise commands.ClientException(f"{error}") from error
