@@ -97,7 +97,22 @@ class LoadSavedStateFromProject:
         )
 
 
-SavedStateAction = Union[LoadSavedStateFromFile, LoadSavedStateFromProject]
+@dataclasses.dataclass(frozen=True)
+class StoreSavedStateToFile:
+    shared_memory_path: str
+
+    def serialize(self) -> Tuple[str, Dict[str, str]]:
+        return (
+            "save_to_file",
+            {
+                "shared_memory_path": self.shared_memory_path,
+            },
+        )
+
+
+SavedStateAction = Union[
+    LoadSavedStateFromFile, LoadSavedStateFromProject, StoreSavedStateToFile
+]
 
 
 @dataclasses.dataclass(frozen=True)
@@ -131,6 +146,9 @@ class Arguments:
     number_of_workers: int = 1
     parallel: bool = True
     profiling_output: Optional[Path] = None
+    python_version: configuration_module.PythonVersion = (
+        configuration_module.PythonVersion(major=3)
+    )
     remote_logging: Optional[RemoteLogging] = None
     saved_state_action: Optional[SavedStateAction] = None
     search_paths: Sequence[configuration_module.SearchPathElement] = dataclasses.field(
@@ -175,6 +193,11 @@ class Arguments:
             "taint_model_paths": self.taint_models_path,
             "debug": self.debug,
             "strict": self.strict,
+            "python_version": {
+                "major": self.python_version.major,
+                "minor": self.python_version.minor,
+                "micro": self.python_version.micro,
+            },
             "show_error_traces": self.show_error_traces,
             "critical_files": [
                 critical_file.serialize() for critical_file in self.critical_files
@@ -253,11 +276,14 @@ def get_saved_state_action(
     start_arguments: command_arguments.StartArguments,
     relative_local_root: Optional[str] = None,
 ) -> Optional[SavedStateAction]:
-    # Loading states from file takes precedence
-    saved_state_file = start_arguments.load_initial_state_from
-    if saved_state_file is not None:
+    saved_state_output_path = start_arguments.save_initial_state_to
+    if saved_state_output_path is not None:
+        return StoreSavedStateToFile(shared_memory_path=saved_state_output_path)
+
+    saved_state_input_path = start_arguments.load_initial_state_from
+    if saved_state_input_path is not None:
         return LoadSavedStateFromFile(
-            shared_memory_path=saved_state_file,
+            shared_memory_path=saved_state_input_path,
             changed_files_path=start_arguments.changed_files_path,
         )
 
@@ -350,6 +376,7 @@ def create_server_arguments(
         number_of_workers=configuration.get_number_of_workers(),
         parallel=not start_arguments.sequential,
         profiling_output=profiling_output,
+        python_version=configuration.get_python_version(),
         remote_logging=remote_logging,
         saved_state_action=None
         if start_arguments.no_saved_state
